
package venice;


/**
 * AppModule holds the whole Venice system together. It has instances
 * of both the <code>ModelStorage</code> and <code>GUI</code>.
 * 
 * <p>AppModule can be constructed in two ways. Either in application mode
 * or in applet mode. When it is constructed in applet mode, it will 
 * resrict its functionality and tell its subsystems do to the same.
 * Applets most often will not be allowed to access the local file system,
 * so commands involving that are left out. The applet mode is meant for
 * viewing models, not editing them.
 * 
 * @author Petteri Kamppuri
 * @version 1.0
 * 
 */
public class AppModule
{
    /**
     * Returns the current <code>GUI</code> instance.
     * Users should not try to save the returned value,
     * but should always query it when needed.
     * 
     * @return current GUI instance
     */
    public GUI getGui()
    {
	return mGui;
    }
    
    /**
     * Returns the current <code>ModelStorage</code> instance.
     * Users should not try to save the returned value,
     * but should always query it when needed.
     * 
     * @return current ModelStorage instance
     */
    public ModelStorage getModelStorage()
    {
	return mModelStorage;
    }
    
    /**
     * Sole constructor. This constructor instantiates and 
     * initializes GUI and ModelStorage. The produced GUI
     * will have different functionality depending on whether
     * AppModule (and GUI) are instantiated as an Applet or not.
     * 
     * @param inIsApplet whether to initialize this instance as an Applet
     */
    public AppModule(boolean inIsApplet)
    {
	Command.setAppModule(this);
	
	mModelStorage = new ModelStorage();
	
	mGui = new GUI(inIsApplet);
    }
    
    /**
     * Private <code>GUI</code> instance.
     */
    private GUI		mGui = null;
    
    /**
     * Private <code>ModelStorage</code> instance.
     */
    private ModelStorage	mModelStorage = null;
}

