package venice;

import java.io.File;
import javax.swing.*;
import javax.swing.filechooser.*;

/** 
 * FileFilter which can be used with filechoosers to filter files so
 * that only files matching an particular extension are listed in
 * the file chooser.
 * <p>
 * This code is based on examples on SUN web site and on FileChooserDemo
 * shipped with JDK.
 *
 * @author Hannu Laurila
 */
public class ExtensionFileFilter extends FileFilter
{    
    private String mExtension;
    private String mDescription;

    public ExtensionFileFilter(String extension, String description)
    {
        mExtension = extension;
        mDescription = description;
    }

    public boolean accept(File f) {
        if (f.isDirectory()) {
            return true;
        }

        String extension = getExtension(f);
        if (extension != null) 
            if (extension.equalsIgnoreCase(mExtension))
                return true;

        return false;
    }
    
    public String getDescription() {
        return mDescription;
    }

    public static String getExtension(File f) {
        String ext = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');
        
        if (i > 0 &&  i < s.length() - 1) {
            ext = s.substring(i+1).toLowerCase();
        }
        return ext;
    }

    /**
     * get String which represents which can be joined to a file
     * with no extension to get 'full name'.
     */
    public String getJoinableExtension()
    {
	return "." + mExtension;
    }
}
