 /**
 * @(#) GUI.java
 */

package venice;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;

//import java.util.NoSuchElementException;
//import java.util.LinkedList;
//import java.util.ListIterator;


/**
 * GUI implements graphic userinterface of venice - UML Visualization tool.
 * User interface has MenuBar with every command supported, ToolBar with
 * selection, panning and zooming buttons and textfield for selection/filtering
 * by type and name.
 *
 * @see MenuToolBar
 *
 * @author Antti Pietarinen
 */

public class GUI extends JFrame
{
    
    int mMode = Visualization.MODE_SELECT;
    int mTmpMode = mMode;
    
    Visualization mVisualization = null;
    
    MenuToolBar  mMenuToolBar;
    
    JPopupMenu mPackagePopup = new JPopupMenu();
    /*Init Package popupmenu*/
    JMenuItem mPDUMP                           = new JMenuItem("Dump");
    JMenuItem mPDepupMenuItem                  = new JMenuItem("Relationships up");
    JMenuItem mPDepdownMenuItem                = new JMenuItem("Relationships down");
    JMenuItem mPShowInformationMenuItem        = new JMenuItem("Show information");      
    JMenu mPStatusMenu                         = new JMenu("Package");
    JMenuItem mPOpenMenuItem                   = new JMenuItem("Open");
    JMenuItem mPClosedMenuItem                 = new JMenuItem("Close");
    JMenuItem mPTransparentMenuItem            = new JMenuItem("Transparent");
    JMenu mPInterfaceMenu                      = new JMenu("Interfaces");
    JCheckBoxMenuItem mPShowRegIntMenuItem     = new JCheckBoxMenuItem("Show required");
    JCheckBoxMenuItem mPShowOffIntMenuItem     = new JCheckBoxMenuItem("Show offered");
    JMenu mPZoomMenu                           = new JMenu("Zoom");
    JMenuItem mPFitToScreenMenuItem            = new JMenuItem("Fit to screen");
    JMenuItem mPZoomToSelectionMenuItem        = new JMenuItem("To selection");
    ButtonGroup mStatusItemsButtonGroup        = new ButtonGroup();

   
    JPopupMenu mComponentPopup                 = new JPopupMenu();
    /*Init Component popupmenu*/
    JMenuItem mCDUMP                           = new JMenuItem("Dump");
    JMenuItem mCDepupMenuItem                  = new JMenuItem("Relationships up");
    JMenuItem mCDepdownMenuItem                = new JMenuItem("Relationships down");
    JMenuItem mCShowInformationMenuItem        = new JMenuItem("Show information");            
    JMenu mCInterfaceMenu                      = new JMenu("Interfaces");
    JCheckBoxMenuItem mCShowRegIntMenuItem     = new JCheckBoxMenuItem("Show required");
    JCheckBoxMenuItem mCShowOffIntMenuItem     = new JCheckBoxMenuItem("Show offered");
    JMenu mCZoomMenu                           = new JMenu("Zoom");
    JMenuItem mCFitToScreenMenuItem            = new JMenuItem("Fit to screen");
    JMenuItem mCZoomToSelectionMenuItem        = new JMenuItem("To selection");

    JPopupMenu mDependencyPopup                = new JPopupMenu();
    /*Init Dependency popupmenu*/
    JMenuItem mDShowInformationMenuItem        = new JMenuItem("Show information");
    JMenu mDZoomMenu                           = new JMenu("Zoom");
    JMenuItem mDFitToScreenMenuItem            = new JMenuItem("Fit to screen");
    JMenuItem mDZoomToSelectionMenuItem        = new JMenuItem("To selection");     

    JPopupMenu mInterfacePopup                 = new JPopupMenu();
    /*Init Interface popupmenu*/
    JMenuItem mIDUMP                            = new JMenuItem("Dump");
    JMenuItem mIShowInformationMenuItem        = new JMenuItem("Show information");
    JCheckBoxMenuItem mIShowContentMenuItem    = new JCheckBoxMenuItem("Show interface content");
    JMenuItem mIHideSelectedMenuItem           = new JMenuItem("Hide interface");
    JMenu mIZoomMenu                           = new JMenu("Zoom");
    JMenuItem mIFitToScreenMenuItem            = new JMenuItem("Fit to screen");
    JMenuItem mIZoomToSelectionMenuItem        = new JMenuItem("To selection");

    
    /**
     * Constructor 
     * @param inIsApplet is user of this class applet or application. True is applet, false application.
     */
    public GUI(boolean inIsApplet)
    {
	if(!inIsApplet)
	    {
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
			    System.exit(0);
			}
		    });
	    }
	setBounds(100, 100, 800, 600);
	setTitle("Venice - UML visualization tool");
	
	/*setVisible(true); Might be nice to set frame visible here */
	
	mMenuToolBar = new MenuToolBar(inIsApplet, this);
	initListeners();
	
	mVisualization = VisualizationFactory.newVisualization();
	mMenuToolBar.getDebugMenu().add(mVisualization.getDebugMenu());
	
	initCanvasPopup();
	
	mVisualization.setToolMode(Visualization.MODE_SELECT);
	getContentPane().add(mVisualization.getSwingComponent());
	setVisible(true);
    }
    
    /**
     * Added by Petteri Kamppuri 29.3.2001 21.56
     * 
     * Gives reference to the current Visualization instance.
     * 
     * @return current visualization instance
     */
    public Visualization getVisualization()
    {
	return mVisualization;
    }
    /**
     *  Updates MenuBar to correspond current selection.
     */

    private void updateMenuBar(){
	VModelElement[] mSelection = mVisualization.getSelection();
	boolean[] table = getInterfaceInformation(mSelection);
	if(mSelection == null){ //no selection made
	    mMenuToolBar.setDefaultMenuBar();
	}
	else if(mSelection.length == 1){ // one VModelElement's selected
	    if(mSelection[0] instanceof VPackage){
		//fixme: need to know status of the package.
		mMenuToolBar.setVPackageMenuBar(table[0],table[1],table[2],table[3]);
	    }
	    else if(mSelection[0] instanceof VComponent){
		mMenuToolBar.setVComponentMenuBar(table[0],table[1],table[2],table[3]);
	    }
	    else if(mSelection[0] instanceof VInterface){
		mMenuToolBar.setVInterfaceMenuBar(getInterfaceVisualization(mSelection));
	    }
	    else{
		mMenuToolBar.setDependencyMenuBar();
	    }	
	} 
	else if(mSelection.length > 1){ // more than one VModelElement selected
	    int mSelectionType = getSelectionType(mSelection);
	    switch(mSelectionType){
	    case 1:                     // All packages
		mMenuToolBar.setVPackageMenuBar(table[0],table[1],table[2],table[3]);
		break;
	    case 2:                     // All components or mix of components and packages elements
		mMenuToolBar.setVComponentMenuBar(table[0],table[1],table[2],table[3]);
		break;
	    case 3:                     // All interfaces
		mMenuToolBar.setVInterfaceMenuBar(getInterfaceVisualization(mSelection));
		break;
	    case 4:                     // All dependencies
		mMenuToolBar.setDependencyMenuBar();
		break;
	    default:                    // Mix of different Model elements
		mMenuToolBar.setDefaultMenuBar();
		break;
	    }
	}
    }
    /**
     *  Updates PopupMenu to correspond current selection.
     */
    private void updatePopupMenu(MouseEvent e){
	VModelElement[] mSelection = mVisualization.getSelection();
	boolean[] table = getInterfaceInformation(mSelection);

	if(mSelection == null){
	    ;
	}
	else if(mSelection.length == 1){
	    if(mSelection[0] instanceof VPackage){
		mPShowRegIntMenuItem.setEnabled(table[1]);
		mPShowRegIntMenuItem.setSelected(table[3]);
		mPShowOffIntMenuItem.setEnabled(table[0]);
		mPShowOffIntMenuItem.setSelected(table[2]);
		mPackagePopup.show( e.getComponent(),e.getX(), e.getY() );
	    }
	    else if(mSelection[0] instanceof VComponent){
		mCShowRegIntMenuItem.setEnabled(table[1]);
                mCShowRegIntMenuItem.setSelected(table[3]);
                mCShowOffIntMenuItem.setEnabled(table[0]);
                mCShowOffIntMenuItem.setSelected(table[2]);
		mComponentPopup.show( e.getComponent(),e.getX(), e.getY() );
	    }
	    else if(mSelection[0] instanceof VInterface){
		mIShowContentMenuItem.setSelected(getInterfaceVisualization(mSelection));
		mInterfacePopup.show( e.getComponent(),e.getX(), e.getY() );
	    }
	    else{
		 mDependencyPopup.show( e.getComponent(),e.getX(), e.getY() );
	    }		    
	}
	else if(mSelection.length > 1){
	    int mSelectionType = getSelectionType(mSelection);
	    switch(mSelectionType){
	    case 1:                     // All packages
		mPShowRegIntMenuItem.setEnabled(table[1]);
		mPShowRegIntMenuItem.setSelected(table[3]);
		mPShowOffIntMenuItem.setEnabled(table[0]);
		mPShowOffIntMenuItem.setSelected(table[2]);
		mPackagePopup.show( e.getComponent(),e.getX(), e.getY() );
		break;
	    case 2:                     // All components or mix of components and packages elements
		mCShowRegIntMenuItem.setEnabled(table[1]);
		mCShowRegIntMenuItem.setSelected(table[3]);
		mCShowOffIntMenuItem.setEnabled(table[0]);
		mCShowOffIntMenuItem.setSelected(table[2]);
		mComponentPopup.show( e.getComponent(),e.getX(), e.getY() );
		break;
	    case 3:                     // All interfaces
		mIShowContentMenuItem.setSelected(getInterfaceVisualization(mSelection));		
		mInterfacePopup.show( e.getComponent(),e.getX(), e.getY() );
		break;
	    case 4:                     // All dependencies
		mDependencyPopup.show( e.getComponent(),e.getX(), e.getY() );
		break;
	    default:                    // Mix of different Model elements
		mDependencyPopup.show( e.getComponent(),e.getX(), e.getY() );
		break;
	    }
	}
    }
    /**
     * Check current selection.
     * @return type of selection. 0 no homogeneous selection made, 1 all packages or components, 
     * 2 all interfaces, 3 something else.
     */
    private int getSelectionType(VModelElement[] mSelection){
	boolean mixOfComponentsAndPackages = false;
	int types = 0;
	VModelElement mElement = null;
	for(int i = 0; i < mSelection.length-1 ; i++){
		if((mSelection[i] instanceof VPackage    && mSelection[i+1] instanceof VPackage)   ||
		   (mSelection[i] instanceof VComponent  && mSelection[i+1] instanceof VComponent) ||
		   (mSelection[i] instanceof VInterface  && mSelection[i+1] instanceof VInterface) ||
		   (mSelection[i] instanceof VRelationship && mSelection[i+1] instanceof VRelationship))
		    ;
		else if((mSelection[i] instanceof VPackage    && mSelection[i+1] instanceof VComponent) ||
			(mSelection[i] instanceof VComponent  && mSelection[i+1] instanceof VPackage) )
		    mixOfComponentsAndPackages = true;
		else
		    return 0;
	}
	if(mixOfComponentsAndPackages)
	    return 2;
	else if(mSelection[0] instanceof VPackage)
	    return 1;
	else if(mSelection[0] instanceof VComponent)
	    return 2;
	else if(mSelection[0] instanceof VInterface)
	    return 3;
	else 
	    return 4;
	
    }
    private boolean getInterfaceVisualization(VModelElement[] mSelection){
	boolean contentShown = false;
	for(int i = 0; i < mSelection.length; i++){
	    if( ((VInterface)mSelection[i]).getViewMode() == VInterface.VIEWMODE_EXPANDED )
		contentShown = true;
	}
	return contentShown;
    }

    private boolean hasRequiredInterfacesShown(VModelElement inE){
	if(inE instanceof ZPackage){
	    if(((ZPackage)inE).getRequiredRealizations().isEmpty() )
		return false;
	    else 
		return true;
	}
	else{
	    if( ((ZVisualLeafModelElement)inE).getRequiredRealizations().isEmpty())
		return false;
	    else
		return true;
	}
    }
    private boolean hasRequiredInterfaces(VModelElement inE){
	if(inE instanceof ZPackage){
            if(((ZPackage)inE).getRequiredInterfaceSubstitutions().isEmpty() )
		return false;
            else
		return true;
	}
	else{
	    if( ((ZVisualLeafModelElement)inE).getRequiredInterfaceSubstitutions().isEmpty())
		return false;
            else
		return true;
	}
    }


    private boolean hasOfferedInterfacesShown(VModelElement inE){
        if(inE instanceof ZPackage){
            if(((ZPackage)inE).getHiddenOfferedRealizations().isEmpty() )
                return false;
            else
                return true;
        }
        else{
            if( ((ZVisualLeafModelElement)inE).getHiddenOfferedRealizations().isEmpty())
                return false;
            else
                return true;
        }
    }

  
    private boolean[] getInterfaceInformation(VModelElement[] mSelection){
	boolean prevOff=false,prevReq=false, showOff = false, showReq = false, hasRequired = false, hasOffered = false;
	LinkedList myRealizationList = null;
	ListIterator myReaIterator = null;
	VRelationship mRealization = null;
	if(mSelection == null)return null;

	for(int i = 0 ; i < mSelection.length; i++){ // for each selectedNode
	    myRealizationList = mSelection[i].getRelationships(null);
	    if(myRealizationList != null){

		myReaIterator = myRealizationList.listIterator(0);
		
		while( myReaIterator.hasNext() ){ // for each Realization
		    try{
			mRealization = (VRelationship) myReaIterator.next();
		    }catch(NoSuchElementException e){
			break;
		    }
		    if( mRealization instanceof VRelationship &&
			mRealization.getSource().getId().equals( mSelection[i].getId()) &&
			mRealization.getDestination() instanceof VInterface ){
			
			if( ((VInterface)mRealization.getDestination()).getInterfaceType() == VInterface.TYPE_OFFERED &&
			    mRealization instanceof VRealization && 
			    !mRealization.getIsRepresentedBySomeoneElse() ){
			    showOff = true;
			}
			if( ((VInterface)mRealization.getDestination()).getInterfaceType() == VInterface.TYPE_OFFERED &&
			    mRealization instanceof VDependency ){
			    hasRequired = true;
			}
			if( ((VInterface)mRealization.getDestination()).getInterfaceType() == VInterface.TYPE_OFFERED &&
			    mRealization instanceof VRealization ){  
			    hasOffered = true;                                 
			}
			
		    }
		}
	    }

	    if(hasRequiredInterfacesShown(mSelection[i]) ){
		showReq = true;
		hasRequired = true;
	    }
	    if(hasRequiredInterfaces(mSelection[i]))
		hasRequired = true;

	    if(hasOfferedInterfacesShown(mSelection[i]) ){
		showOff = false;
		hasOffered = true;
	    }

	    if(i == 0){
		prevOff=showOff;
		prevReq=showReq;
	    }
	   
	    if(prevOff != showOff)
		prevOff = false;
	    if(prevReq != showReq)
		prevReq = false;
	    if(prevOff && prevReq)
		break;

	}

	return new boolean[]{hasOffered, hasRequired, showOff, showReq};
    }

    private void initListeners(){
	mMenuToolBar.zoomVol.addChangeListener(new ChangeListener() 
	    {
		public void stateChanged(ChangeEvent e) {
		    JSlider source = (JSlider)e.getSource();
		    if (!source.getValueIsAdjusting()) {
			int fps = (int)source.getValue();
			mVisualization.setZoomVolume(fps);
		    }
		}
	    }
					       );
	mMenuToolBar.pan.addActionListener(new ActionListener() 
	    {
		public void actionPerformed(ActionEvent ae) {
		    mMode = Visualization.MODE_PAN;
		    mTmpMode = mMode;
		    mVisualization.setToolMode(Visualization.MODE_PAN);
		}
	    }
					   );
	mMenuToolBar.zoom.addActionListener(new ActionListener() 
	    {
		public void actionPerformed(ActionEvent ae) {
		    mMode = Visualization.MODE_ZOOM;		
		    mTmpMode = mMode;
		    mVisualization.setToolMode(Visualization.MODE_ZOOM);
		}
	    }
					    );
	mMenuToolBar.select.addActionListener(new ActionListener() 
	    {
		public void actionPerformed(ActionEvent ae) {
		    mMode = Visualization.MODE_SELECT;
		    mTmpMode = mMode;
		    mVisualization.setToolMode(Visualization.MODE_SELECT);
		}
	    }
					      );
	mMenuToolBar.mFileMenu.addMenuListener(new MenuListener()
	    {
		public void menuCanceled(MenuEvent e){}
		public void menuSelected(MenuEvent e){updateMenuBar();}
		public void menuDeselected(MenuEvent e){}
	    }
							 );
	mMenuToolBar.mEditMenu.addMenuListener(new MenuListener()
	    {
		public void menuCanceled(MenuEvent e){}
                public void menuSelected(MenuEvent e){updateMenuBar();}
                public void menuDeselected(MenuEvent e){}
	    }
							 );
	mMenuToolBar.mViewMenu.addMenuListener(new MenuListener()
	    {
                public void menuCanceled(MenuEvent e){}
                public void menuSelected(MenuEvent e){updateMenuBar();}
                public void menuDeselected(MenuEvent e){}
	    }
							);
	mMenuToolBar.mZoomMenu.addMenuListener(new MenuListener()
	    {
                public void menuCanceled(MenuEvent e){}
                public void menuSelected(MenuEvent e){updateMenuBar();}
                public void menuDeselected(MenuEvent e){}
	    }
							 );		
	mMenuToolBar.mSelectionMenu.addMenuListener(new MenuListener()
	    {
                public void menuCanceled(MenuEvent e){}
                public void menuSelected(MenuEvent e){updateMenuBar();}
                public void menuDeselected(MenuEvent e){}
	    }
							      );
	mMenuToolBar.mFilterMenu.addMenuListener(new MenuListener()
	    {
                public void menuCanceled(MenuEvent e){}
                public void menuSelected(MenuEvent e){updateMenuBar();}
                public void menuDeselected(MenuEvent e){}
	    }
							   );
    }
    private void initCanvasPopup()
    {
	/*Package Popup*/
	mPackagePopup.add(mPDUMP);
	mPackagePopup.add(mPDepupMenuItem);  
	mPackagePopup.add(mPDepdownMenuItem);
	//	mPackagePopup.add(mPShowInformationMenuItem);
	mPackagePopup.add(mPStatusMenu);
	mPackagePopup.add(mPInterfaceMenu);
	mPInterfaceMenu.add(mPShowRegIntMenuItem);
    	mPInterfaceMenu.add(mPShowOffIntMenuItem);
	mPackagePopup.add(mPZoomMenu);
	mPZoomMenu.add(mPFitToScreenMenuItem);
	mPZoomMenu.add(mPZoomToSelectionMenuItem);

//    	mStatusItemsButtonGroup.add(mPOpenMenuItem);
//    	mStatusItemsButtonGroup.add(mPClosedMenuItem);
//    	mStatusItemsButtonGroup.add(mPTransparentMenuItem);
    	mPStatusMenu.add(mPOpenMenuItem);
    	mPStatusMenu.add(mPClosedMenuItem);
    	mPStatusMenu.add(mPTransparentMenuItem);
 
	//	mPackagePopup.add(mPZoomToSelectionMenuItem);
	//	mPackagePopup.add(mPFilterMenuItem);	
	/*Component Popup*/
	mComponentPopup.add(mCDUMP);
	mComponentPopup.add(mCDepupMenuItem);
	mComponentPopup.add(mCDepdownMenuItem);
	//	mComponentPopup.add(mCShowInformationMenuItem);               
	mComponentPopup.add(mCInterfaceMenu);
	mCInterfaceMenu.add(mCShowRegIntMenuItem);
	mCInterfaceMenu.add(mCShowOffIntMenuItem);
	mComponentPopup.add(mCZoomMenu);
	mCZoomMenu.add(mCFitToScreenMenuItem);
	mCZoomMenu.add(mCZoomToSelectionMenuItem);

	/*Interface Popup*/
	//	mInterfacePopup.add(mIShowInformationMenuItem);
	mInterfacePopup.add(mIDUMP);
	mInterfacePopup.add(mIShowContentMenuItem);
	//	mInterfacePopup.add(mIHideSelectedMenuItem);
	mInterfacePopup.addSeparator();
	mInterfacePopup.add(mIZoomMenu);
	mIZoomMenu.add(mIFitToScreenMenuItem);
	mIZoomMenu.add(mIZoomToSelectionMenuItem);

	/*Dependency Popup*/
	//	mDependencyPopup.add(mDShowInformationMenuItem);
	mDependencyPopup.add(mDZoomMenu);
	mDZoomMenu.add(mDFitToScreenMenuItem);
	mDZoomMenu.add(mDZoomToSelectionMenuItem);

	mVisualization.getSwingComponent().addMouseListener (new MouseAdapter() {
		public void mousePressed( MouseEvent e )
		{ checkForTriggerEvent( e ); } 
		public void mouseReleased( MouseEvent e )
		{ checkForTriggerEvent( e ); } 
		private void checkForTriggerEvent( MouseEvent e )
		{	
		    if ( e.isPopupTrigger() ) {
			if( mMode != Visualization.MODE_ZOOM ){
			    updatePopupMenu(e);
			}  
		    }
		}
	    }
							     );
	mPDUMP.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
		    VModelElement[] mSel = mVisualization.getSelection();
		    for(int i = 0; i< mSel.length; i++)
			mSel[i].debugDump();
		}
	    }
				 );
	mCDUMP.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
                    VModelElement[] mSel = mVisualization.getSelection();
		    for(int i = 0; i< mSel.length; i++)
			mSel[i].debugDump();
		}
	    }
				 );
	mIDUMP.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent ae){
                    VModelElement[] mSel = mVisualization.getSelection();
		    for(int i = 0; i< mSel.length; i++)
			mSel[i].debugDump();
		}
	    }
				 );
	mPOpenMenuItem.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ae) {
                    new OpenPackageCommand().execute();
                }
            }
					 );
	mPClosedMenuItem.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ae) {
                    new ClosePackageCommand().execute();
                }
            }
					   );
	mPTransparentMenuItem.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ae) {
                    new MakePackageTransparentCommand().execute();
                }
            }
						);
	mPDepupMenuItem.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ae) {
                    new MoveRelationshipsUpCommand().execute();
                }
            }
					  );
        mPDepdownMenuItem.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ae) {
                    new MoveRelationshipsDownCommand().execute();
                }
            }
					    );
        mCDepupMenuItem.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ae) {
                    new MoveRelationshipsUpCommand().execute();
                }
            }
					  );
        mCDepdownMenuItem.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ae) {
                    new MoveRelationshipsDownCommand().execute();
                }
            }
					    );
	/*****************************************/
	mPShowRegIntMenuItem.addActionListener(new ActionListener()
	    {
                public void actionPerformed(ActionEvent ae) {
		    if( !mPShowRegIntMenuItem.isEnabled() )return;
                    if( !mPShowRegIntMenuItem.isSelected() ){
			new HideRequiredInterfacesCommand().execute();
		    }
		    else{
			new ShowRequiredInterfacesCommand().execute();
		    }
                }
            }
					       );
	mPShowOffIntMenuItem.addActionListener(new ActionListener()
            {
		public void actionPerformed(ActionEvent ae) {
		    if( !mPShowOffIntMenuItem.isEnabled() )return;
                    if( !mPShowOffIntMenuItem.isSelected() ){
			new HideOfferedInterfacesCommand().execute();
                    }
                    else{
                        new ShowOfferedInterfacesCommand().execute();
                    }
		}
            }
                                               );
	mCShowRegIntMenuItem.addActionListener(new ActionListener()
	    {
                public void actionPerformed(ActionEvent ae) {
                    if( !mCShowRegIntMenuItem.isEnabled() )return;
                    if( !mCShowRegIntMenuItem.isSelected() ){
			new HideRequiredInterfacesCommand().execute();
                    }
                    else{
                        new ShowRequiredInterfacesCommand().execute();
                    }
                }
            }
					       );
	mCShowOffIntMenuItem.addActionListener(new ActionListener()
            {
		public void actionPerformed(ActionEvent ae) {
                    if( !mCShowOffIntMenuItem.isEnabled() )return;
                    if( !mCShowOffIntMenuItem.isSelected() ){
			new HideOfferedInterfacesCommand().execute();
                    }
                    else{
                        new ShowOfferedInterfacesCommand().execute();
                    }
		}
            }
                                               );
	/********************************/

	mPZoomToSelectionMenuItem.addActionListener(new ActionListener()
	    {
                public void actionPerformed(ActionEvent ae) {
		    new ZoomToSelectionCommand().execute();
                }
            }
						    );
	mCZoomToSelectionMenuItem.addActionListener(new ActionListener()
	    {
                public void actionPerformed(ActionEvent ae) {
		    new ZoomToSelectionCommand().execute();
                }
            }
						    );
	mIZoomToSelectionMenuItem.addActionListener(new ActionListener()
	    {
                public void actionPerformed(ActionEvent ae) {
		    new ZoomToSelectionCommand().execute();
                }
            }
						    );
	mPFitToScreenMenuItem.addActionListener(new ActionListener()
	    {
                public void actionPerformed(ActionEvent ae) {
		    new FitToScreenCommand().execute();
                }
            }
	);
	mCFitToScreenMenuItem.addActionListener(new ActionListener()
	    {
                public void actionPerformed(ActionEvent ae) {
		    new FitToScreenCommand().execute();
                }
            }
						);
	mIFitToScreenMenuItem.addActionListener(new ActionListener()
	    {
                public void actionPerformed(ActionEvent ae) {
		    new FitToScreenCommand().execute();
                }
            }
						);
	mDZoomToSelectionMenuItem.addActionListener(new ActionListener()
	    {
                public void actionPerformed(ActionEvent ae) {
		    new ZoomToSelectionCommand().execute();
                }
            }
						    );
	mDFitToScreenMenuItem.addActionListener(new ActionListener()
	    {
                public void actionPerformed(ActionEvent ae) {
		    new FitToScreenCommand().execute();
                }
            }
 						);

        mIShowContentMenuItem.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent ae){
		    if(!mIShowContentMenuItem.isEnabled())return;
		    if( !mIShowContentMenuItem.isSelected() ){
			new HideInterfaceContentCommand().execute();
		    }
		    else{
			new ShowInterfaceContentCommand().execute();
		    }
                }
	    });

	addKeyListener(new KeyListener()
	    {
		public void keyPressed(KeyEvent e){
		    if(e.getKeyCode() == KeyEvent.VK_CONTROL && mMode != Visualization.MODE_ZOOM){
			mTmpMode = mMode;
			mMode = Visualization.MODE_ZOOM;
			mVisualization.setToolMode(Visualization.MODE_ZOOM);
			mMenuToolBar.zoom.setSelected(true);
		    }
		}
		public void keyReleased(KeyEvent e){
		    if(e.getKeyCode() == KeyEvent.VK_CONTROL && mMode == Visualization.MODE_ZOOM){
			switch(mTmpMode){
			case Visualization.MODE_ZOOM:
			    break;
			case Visualization.MODE_SELECT:
			    mVisualization.setToolMode(Visualization.MODE_SELECT);
			    mMenuToolBar.select.setSelected(true);
			    break;
			case Visualization.MODE_PAN:
			    mVisualization.setToolMode(Visualization.MODE_PAN);
			    mMenuToolBar.pan.setSelected(true);
			    break;
			}
			mMode = mTmpMode;
		    }
		}
		public void keyTyped(KeyEvent e){}
	    }
		       );
    }
}
