package venice;

import java.io.*;
import java.util.LinkedList;
import java.io.OutputStream;
import java.io.InputStream;

import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.SAXException;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;

/**
 * This class is used to read GXL-files.
 *
 * @author Sami Ilonen
 */

class GXLReader extends DefaultHandler {

	static private Writer out;

	// Flags used in parsing
	private boolean node = false;
	private boolean edge = false;
	private boolean attr = false;
	private boolean attrName = false;
	private boolean attrType = false;
	private boolean attrOperation = false;
	private boolean str = false; 

	private Node tempNode;
	private Edge tempEdge;

	private LinkedList nodeList;
	private LinkedList edgeList;

	public GXLReader(LinkedList nodes, LinkedList edges) {

		this.tempNode = new Node();
		this.tempEdge = new Edge();
		this.nodeList = nodes;
		this.edgeList = edges;
	}

	protected void constructMElement(Node inNode) {

		if (inNode.getNodeType().toLowerCase().equals("schema.gxl#package")) {
			nodeList.addLast(new MPackage (inNode.getID(),inNode.getName(),inNode.getType()));
		}
		else if (inNode.getNodeType().toLowerCase().equals("schema.gxl#component")) {
			nodeList.addLast(new MComponent (inNode.getID(),inNode.getName(),inNode.getType()));
		}
		else if (inNode.getNodeType().toLowerCase().equals("schema.gxl#interface")) {
			nodeList.addLast(new MInterface (inNode.getID(),inNode.getName(),inNode.getType(),inNode.getOperations()));
		}
	}

	protected void constructMRelationship (Edge inEdge) {

		if (inEdge.getEdgeType().equals("schema.gxl#realize")) {
			edgeList.addLast(new MRealization (inEdge.getID(),inEdge.getType(),inEdge.getFrom(),inEdge.getTo()));
		}
		else if (inEdge.getEdgeType().equals("schema.gxl#dependency")) {
			edgeList.addLast(new MDependency (inEdge.getID(),inEdge.getType(),inEdge.getFrom(),inEdge.getTo()));
		}
		else if (inEdge.getEdgeType().equals("schema.gxl#contain")) {
			edgeList.addLast(new MContainment (inEdge.getID(),inEdge.getType(),inEdge.getFrom(),inEdge.getTo()));
		}
		else if (inEdge.getEdgeType().equals("schema.gxl#inheritance")) {
			edgeList.addLast(new MGeneralization (inEdge.getID(),inEdge.getType(),inEdge.getFrom(),inEdge.getTo()));
		}		
	}

	/* DefaultHandler methods: */

	public void StartDocument() throws SAXException {
	}

	public void endDocument() throws SAXException {

	}	
	public void startElement (String uri, String localName, String qName, Attributes attributes) throws SAXException {
		
		if (localName.toLowerCase().equals("node")) {
			setNode(true);
			for (int i = 0; i < attributes.getLength();i++) {
				if (attributes.getLocalName(i).equals("id")) {
					tempNode.setID(attributes.getValue(i));
				}
			}
		} 
		else if (localName.toLowerCase().equals("edge")) {
			setEdge(true);
			for (int i = 0; i < attributes.getLength(); i++) {
				if (attributes.getLocalName(i).equals("id")) {
					tempEdge.setID(attributes.getValue(i));
				}
				else if (attributes.getLocalName(i).equals("from")) {
					tempEdge.setFrom(attributes.getValue(i));
				}
				else if (attributes.getLocalName(i).equals("to")) {
					tempEdge.setTo(attributes.getValue(i));
				}
			}
		}

		/* If parsing a Node */

		if (isNode()) {

			if (localName.toLowerCase().equals("type")) {
				String attrLocalName = attributes.getLocalName(0);
				String attrValue = attributes.getValue(0);
				if (attrLocalName.toLowerCase().equals("href")) {
					tempNode.setNodeType(attrValue);
				}
			} 
		
			else if (localName.toLowerCase().equals("attr")) {
				setAttr(true);
				String attrLocalName = attributes.getLocalName(0);
				String attrAttr = attributes.getValue(0);

				if (attrLocalName.toLowerCase().equals("name")) {
					if (attrAttr.toLowerCase().equals("name")) {
						setAttrName(true); // seuraava string on Name 
					} 
					else if (attrAttr.toLowerCase().equals("type")) {
						setAttrType(true); // seuraava string on Type
					}
					else if (attrAttr.toLowerCase().equals("operation")) {
						setAttrOperation(true); // seuraava string on Operation
					}
				}
			}
			
			else if (localName.toLowerCase().equals("string")) {
				setStr(true);
			}
		}

		/* If parsing an Edge */

		else if (isEdge()) {
			if (localName.toLowerCase().equals("type")) {
				String attrLocalName = attributes.getLocalName(0);
				String attrValue = attributes.getValue(0);
				if (attrLocalName.toLowerCase().equals("href")) {
					tempEdge.setEdgeType(attrValue);
				}
			}
			else if (localName.toLowerCase().equals("attr")) {
				setAttr(true);
				String attrLocalName = attributes.getLocalName(0);
				String attrAttr = attributes.getValue(0);

				if (attrLocalName.toLowerCase().equals("name")) {
					if (attrAttr.toLowerCase().equals("type")) {
						setAttrType(true); // seuraava string on Type
					}
				}
			}			
		}
    }

	public void endElement (String uri, String localName, String qName) throws SAXException {

		if (localName.toLowerCase().equals("node")) {
			this.constructMElement(tempNode);
			tempNode = new Node(); 
			setNode(false);

		} else if (localName.toLowerCase().equals("edge")) {
			this.constructMRelationship(tempEdge);
			tempEdge = new Edge();
			setEdge(false);
		}

		/* If parsing a Node */

		if (isNode()) {

			if (localName.toLowerCase().equals("attr")) {
				setAttr(false);
				setAttrOperation(false);
				setAttrName(false);
				setAttrType(false);
			}
			
			else if (localName.toLowerCase().equals("string")) {
				setStr(false);
			}
		}

		/* If parsing an Edge */

		else if (isEdge()) {
			if (localName.toLowerCase().equals("attr")) {
				setAttr(false);
				setAttrType(false);
			}
		}
    }

	public void characters (char ch [], int start, int length) throws SAXException {

		if (isNode()) {
			if (isAttrName()) {
				tempNode.setName( new String (ch, start, length));
			}
			else if (isAttrType()) {
				tempNode.setType( new String (ch, start, length));
			}
			else if (isAttrOperation()) {
				tempNode.addOperation( new String (ch, start, length));
			}
		}
		if (isEdge()) {
			if (isAttrType()) {
				tempEdge.setType( new String(ch, start, length));
			}
		}
		//		System.out.println(new String(ch, start, length));
	}
	
	public InputSource resolveEntity (String publicId, String systemId) throws SAXException {
		InputStream dtdInputStream = this.getClass().getClassLoader().getResourceAsStream("venice/gxl.dtd");
        return new InputSource(dtdInputStream);		
	}

	// Methods to set the boolean flags
	private void setNode (boolean val) {
		this.node = val;
	}
	private void setEdge (boolean val) {
		this.edge = val;
	}
	private void setAttr (boolean val) {
		this.attr = val;
	}
	private void setAttrName (boolean val) {
		this.attrName = val;
	}
	private void setAttrType (boolean val) {
		this.attrType = val;
	}
	private void setAttrOperation (boolean val) {
		this.attrOperation = val;
	}
	private void setStr (boolean val) {
		this.str = val;
	}

	// Methods to check the flags.
	private boolean isNode () {
		return node;
	}
	private boolean isEdge () {
		return edge;
	}
	private boolean isAttr () {
		return attr;
	}
	private boolean isAttrName () {
		return attrName;
	}
	private boolean isAttrType () {
		return attrType;
	}
	private boolean isAttrOperation() {
		return attrOperation;
	}
	private boolean isStr() {
		return str;
	}
}
