/**
 * @(#) HideOfferedInterfacesCommand.java
 */

package venice;

import java.util.NoSuchElementException;
import java.util.LinkedList;
import java.util.ListIterator;

/**
 * This command hides interfaces offered by this Element.
 * Interface and relationship to it are removed from visualization
 * and new relationship is generated to substitute interface and 
 * relationships.
 * 
 * @author	Antti Pietarinen
 * @version	1.1
 * 
 */
public class HideOfferedInterfacesCommand extends Command
{
	/**
	 * Only method which does the hiding.
	 * 
	 */
	public void execute()
	{
		mModel		= getAppModule().getModelStorage();
		mVisualization	= getAppModule().getGui().getVisualization();
		
		/* Current selection from visualisation subsystem*/
		VModelElement[] mSelection = mVisualization.getSelection();
		VRelationship mRealization  = null;
		VRelationship mDependency = null;

		LinkedList myRealizationList;
		LinkedList myDependencyList;
		ListIterator myReaIterator;
		ListIterator myDepIterator;
		
		VInterface mOffInterface     = null;
		VInterface mTmpInterface     = null;
		MElement mOrgInterface       = null;
		VRealization mNewRealization = null;
		VDependency mNewDependency   = null;
		VDependency mSubsDependency  = null;
		
		for(int i = 0 ; i < mSelection.length; i++){ // for each selectedNode
		    myRealizationList = mSelection[i].getRelationships(null);
		    
		    if(myRealizationList.isEmpty())
			return;
		    
		    myReaIterator = myRealizationList.listIterator(0);
		    
		    while( myReaIterator.hasNext() ){ // for each Relationship
			try{
			    mRealization = (VRelationship) myReaIterator.next();
			}catch(NoSuchElementException e){
			    break;
			}
			if( mRealization instanceof VRealization &&
			    mRealization.getSource().getId().equals( mSelection[i].getId() ) &&
			    ((VInterface)mRealization.getDestination()).getInterfaceType() == VInterface.TYPE_OFFERED ){
			    
      			    mOffInterface = (VInterface)mRealization.getDestination();
			    
                            mOffInterface.setVisible(false);
			    mOffInterface.setHiddenRelationships(mRealization);
			    mRealization.setIsRepresentedBySomeoneElse(true);
			    
			    if(mSelection[i] instanceof ZPackage)
				((ZPackage)mSelection[i]).addToHiddenOfferedRealizations(mRealization);
			    else
				((ZVisualLeafModelElement)mSelection[i]).addToHiddenOfferedRealizations(mRealization);
			    
			    mVisualization.removeVRelationship(mRealization); //remove original realization from Visualization
			    

			    myDependencyList = mOffInterface.getRelationships(null);
			    if(myDependencyList == null) break;
			    myDepIterator = myDependencyList.listIterator(0);
			    while( myDepIterator.hasNext() ){ // for each incoming Dependency
				try{
				    mDependency = (VRelationship) myDepIterator.next();
				}catch(NoSuchElementException e){
				    break;
				}
				if(mDependency instanceof VRealization)break;

				mNewDependency = mVisualization.newVDependency(mDependency.getSource().getId() +"(subsDep)"+mSelection[i].getId() );
				
				mNewDependency.setSource( mDependency.getSource() );
				mOffInterface.setHiddenRelationships( mDependency );
				
				mNewDependency.setDestination(mSelection[i]);
				mNewDependency.setType( mDependency.getType() );

				mVisualization.addVRelationshipWithoutAddingOriginalReference(mNewDependency);
				mOffInterface.addSubsDeps(mNewDependency);
				
				if( mDependency.getSource() instanceof ZPackage)
                                    ((ZPackage)mDependency.getSource()).addToRequiredInterfaceSubstitutions(mNewDependency);
                                else
                                    ((ZVisualLeafModelElement)mDependency.getSource()).addToRequiredInterfaceSubstitutions(mNewDependency);
				
				if(mSelection[i] instanceof ZPackage)
				    ((ZPackage)mSelection[i]).addToOfferedRealizationSubstitutions(mNewDependency);
				else
				    ((ZVisualLeafModelElement)mSelection[i]).addToOfferedRealizationSubstitutions(mNewDependency);
				
				mDependency.setIsRepresentedBySomeoneElse(true);
                                mVisualization.removeVRelationship( mDependency );

			    }
			}
		    
		    }
		    /* for debuggin
		    if(mSelection[i] instanceof ZPackage)
			myDependencyList = ((ZPackage)mSelection[i]).getHiddenOfferedRealizations();
		    else
			myDependencyList = ((ZVisualLeafModelElement)mSelection[i]).getHiddenOfferedRealizations();
		    myDepIterator = myDependencyList.listIterator(0);
		    while(myDepIterator.hasNext()){
			mDependency = (VRelationship)myDepIterator.next();
			System.out.println(mDependency.getDestination().getId());
		    }
		    */
		}
	}
    
    
    /**
     * Private variable to hold the ModelStorage instance that is needed
     * in various places in the recursive action of this object. 
     */
    private ModelStorage  mModel		= null;
    
    /**
     * Private variable to hold the Visualization instance that is needed
     * in various places in the recursive action of this object. 
     */
    private Visualization mVisualization	= null;

}
