/**
 * @(#) HideRequiredInterfacesCommand.java
 */

package venice;

import java.util.NoSuchElementException;
import java.util.LinkedList;
import java.util.ListIterator;

/**
 * This command hides required interfaces generated by Venice.
 * Required interface and its realization is removed from 
 * visualization.
 * 
 * @author	Antti Pietarinen
 * @version	1.1
 * 
 */
public class HideRequiredInterfacesCommand extends Command
{
	/**
	 * Only method which does the job.
	 * 
	 */
	public void execute()
	{
		mModel		= getAppModule().getModelStorage();
		mVisualization	= getAppModule().getGui().getVisualization();
		
		/* Current selection from visualisation subsystem*/
		VModelElement[] mSelection = mVisualization.getSelection();
		VRelationship mRelationship  = null;

		LinkedList myList;
		ListIterator myIterator;
                LinkedList mySubsList;
                ListIterator mySubsIterator;

		
		VInterface mReqInterface     = null;
			
		for(int i = 0 ; i < mSelection.length; i++){ // for each selectedNode
		    if(mSelection[i] instanceof ZPackage)
			myList = ((ZPackage)mSelection[i]).getRequiredRealizations();
		    else
			myList = ((ZVisualLeafModelElement)mSelection[i]).getRequiredRealizations();

		    if(myList.isEmpty())break;
		    myIterator = myList.listIterator(0);

		    while( myIterator.hasNext() ){ // for each Required Realization
			try{
			    mRelationship = (VRelationship) myIterator.next();
			}catch(NoSuchElementException e){
			    break;
			}
			mReqInterface = (VInterface)mRelationship.getDestination();

			if(mReqInterface.getVParent() != null)
			    mReqInterface.getVParent().removeVChild(mReqInterface);
			else
			    mVisualization.getRootPackage().removeVChild(mReqInterface);

			mVisualization.removeVRelationship(mRelationship);
		    }
		    if(mSelection[i] instanceof ZPackage)
                        ((ZPackage)mSelection[i]).clearRequiredRealizations();
                    else
                        ((ZVisualLeafModelElement)mSelection[i]).clearRequiredRealizations();

		}
	}


	/**
	 * Private variable to hold the ModelStorage instance that is needed
	 * in various places in the recursive action of this object. 
	 */
	private ModelStorage  mModel		= null;
	
	/**
	 * Private variable to hold the Visualization instance that is needed
	 * in various places in the recursive action of this object. 
	 */
	private Visualization mVisualization	= null;

}
