package venice;

import java.io.*;
import javax.swing.*;

/**
 * Implements .venice-loading from a file.
 *
 * @author Hannu Laurila
 */
public class LoadFileCommand extends LoadCommand
{
    private String mSuppliedFilename;

    /**
     * constructs a command which will ask user for the
     * file name.
     */
    public LoadFileCommand()
    {
	this(null);
    }

    /**
     * constructs a command which uses the supplied file name and does
     * not ask user for the name. if null is given, file name will be
     * asked.
     */
    public LoadFileCommand(String inFilename)
    {
	mSuppliedFilename = inFilename;
    }

    /**
     * Opens file chooser dialog and opens the file specified by user 
     * for reading.
     */
    protected InputStream getInputStream()
    {
	File myFile;

	if (mSuppliedFilename != null) {
	    // Filename was supplied in constructor. Try that file.
	    myFile = new File(mSuppliedFilename);
	} else {
	    // filename was not supplied. ask for it.
	    final JFileChooser myFC = new JFileChooser();
	    myFC.addChoosableFileFilter(new VeniceFileFilter());
	    int returnVal = myFC.showOpenDialog(null);
	    
	    if(returnVal != JFileChooser.APPROVE_OPTION)
		return null;
	    
	    myFile = myFC.getSelectedFile();
	}
	
	try {
	    InputStream myIs = new FileInputStream(myFile);
	    return myIs;
	}
	catch (IOException myEx) {
	    Util.handleException(myEx);
	    return null;
	}
    }
}
