package venice;

/**
 * This class provides an abstraction for all relationships used in the model
 * @author Sami Ilonen
 */

public abstract class MRelationship {

	private String mId;
	private String mType;
	private String mSource;
	private String mDestination;

	public MRelationship(String inId, String inType, String inSrc, String inDest) {
		this.mId = inId;
		this.mType = inType;
		this.mSource = inSrc;
		this.mDestination = inDest;
	}

	/**
	 * Returns the id of this relationship.
	 * @return Returns the id of this relationship.
	 */

	public String getId() {
		return mId;
	}

	/**
	 * Returns the source ModelElement for this relationship.
	 *
	 * @return Returns the source of this relationship.
	 *
	 */

	public String getSource() {
		return mSource;
	}

	/**
	 * Returns the destination ModelElement for this relationship.
	 *
	 * @return Returns the destination of the relationship.
	 *
	 */

	public String getDestination() {
		return mDestination;
	}

	/**
	 * Returns the type of this relationship as String.
	 *
	 * @return Returns the type of this relationship.
	 *
	 */

	public String getType() {
		return mType;
	}

	/**
	 * Returns the XML-representation of this relationship as String.
	 *
	 * @return Returns the XML-representation as String.
	 *
	 */

	abstract public String toXML();
}
