/**
 * @(#) MoveRelationshipsUpCommand.java
 */

package venice;

import java.util.*;


/**
 * Moves relationships up for all selected nodes.
 * 
 * @author	Petteri Kamppuri
 * @version	1.1
 * 
 */
public class MoveRelationshipsUpCommand extends Command
{
    /**
     * <pre>
     * for each child as myChild
     *        for each relationship of myChild and its children as myRel
     *                if not (myRel.src isDescentOf getParent() and myRel.dst isDescentOf getParent())
     *                        myRel.hide
     *                        getParent().addToCompositeRelationship(myRel with dst or src moved to parent)
     * </pre>
     */
    public void execute()
    {
	mModel			= getAppModule().getModelStorage();
	mVisualization	= getAppModule().getGui().getVisualization();
	
	/* Get current selection from visualisation subsystem*/
	VModelElement[] mySelection = mVisualization.getSelection();
	
	if(mySelection != null) {
	    // Iterate through selected elements
	    for(int i = 0; i < mySelection.length; ++i) {
		// Find parent in visualization for selected element
		VPackage myParent = mySelection[i].getVParent();
		
		if(myParent == null)
		    continue;
		
		LinkedList myListOfRelationships = mySelection[i].getRelationships(myParent);
		
		if(myListOfRelationships != null) {
		    ListIterator myIterator = myListOfRelationships.listIterator(0);
		    
		    while(myIterator.hasNext()) {
			VRelationship myRel = (VRelationship) myIterator.next();
			
			mVisualization.removeVRelationship(myRel);
			ClosePackageCommand.addRelationshipToCompRelListNotifyingOtherEnd(myParent, myRel);
		    }
		}
	    }
	}
	
    }
    
    /**
     * Private variable to hold the ModelStorage instance that is needed
     * in various places in the recursive action of this object. 
     */
    private ModelStorage  mModel		= null;
    
    /**
     * Private variable to hold the Visualization instance that is needed
     * in various places in the recursive action of this object. 
     */
    private Visualization mVisualization	= null;
}



