/**
 * @(#) ShowOfferedInterfacesCommand.java
 */

package venice;

import java.util.NoSuchElementException;
import java.util.LinkedList;
import java.util.ListIterator;

/**
 * This command show interfaces that current element 
 * offers. Interface information is recieved from GXL model.
 * 
 * @author	Antti Pietarinen
 * @version	1.1
 * 
 */
public class ShowOfferedInterfacesCommand extends Command
{
	/**
	 * Only method that executes this command.
	 * 
	 */
    public void execute()
	{
	        /** Current Model */
		mModel		= getAppModule().getModelStorage();
		/** Current Visualization*/
		mVisualization	= getAppModule().getGui().getVisualization();
		
		/** Current selection from visualisation subsystem*/
		VModelElement[] mSelection = mVisualization.getSelection();
		VRelationship mRealization  = null;
		MDependency mDependency = null;

		LinkedList myRealizationList;
		LinkedList myDependencyList;
		ListIterator myReaIterator;
		ListIterator myDepIterator;
		LinkedList mTemp = new LinkedList();
		ListIterator mTempList;
		
		VInterface mOffInterface     = null;
		VInterface mTmpInterface     = null;
		MElement mOrgInterface       = null;
		VRealization mNewRealization = null;
		VDependency mNewDependency   = null;
		VRelationship mHiddenRelationship  = null;
		boolean noDep = true;		

		for(int i = 0 ; i < mSelection.length; i++){ // for each selectedNode
		    if(mSelection[i] instanceof ZPackage)
			myRealizationList = ((ZPackage)mSelection[i]).getHiddenOfferedRealizations();
		    else
			myRealizationList = ((ZVisualLeafModelElement)mSelection[i]).getHiddenOfferedRealizations();
		    if(myRealizationList == null)
			return;
		    myReaIterator = myRealizationList.listIterator(0);
		    System.out.println(myRealizationList.size());
		    while( myReaIterator.hasNext() ){ // for each Hidden offered Realization
                        try{
                            mRealization = (VRelationship) myReaIterator.next();
                        }catch(NoSuchElementException e){
                            break;
                        }
			System.out.println("+"+mRealization.getDestination().getId());
			mVisualization.addVRelationshipWithoutAddingOriginalReference(mRealization);
			mRealization.setIsRepresentedBySomeoneElse(false);
			mOffInterface = (VInterface)mRealization.getDestination();
			mOffInterface.setVisible(true);

			myDependencyList = mOffInterface.getHiddenRelationships();
			if(!myDependencyList.isEmpty()){
			    myDepIterator = myDependencyList.listIterator(0);
			    while( myDepIterator.hasNext() ){ // for each Hidden dependency
				try{
				    mHiddenRelationship = (VRelationship) myDepIterator.next();
				}catch(NoSuchElementException e){
				    break;
				}
				if( mHiddenRelationship instanceof VDependency ){
				    mVisualization.addVRelationshipWithoutAddingOriginalReference(mHiddenRelationship);
				    mHiddenRelationship.setIsRepresentedBySomeoneElse(false);
				    //mTemp.add(mHiddenRelationship.getSource());
				}
			    }
			}
			myDependencyList = mOffInterface.getSubsDeps();
			if(!myDependencyList.isEmpty()){
                            myDepIterator = myDependencyList.listIterator(0);
                            while( myDepIterator.hasNext() ){ // for each substitute dependency
				try{
                                    mHiddenRelationship = (VRelationship) myDepIterator.next();
				}catch(NoSuchElementException e){
                                    break;
				}
                                if( mHiddenRelationship instanceof VDependency ){// poistetaan subsdep
                                    mVisualization.removeVRelationship(mHiddenRelationship);
				    if(mHiddenRelationship.getSource() instanceof ZPackage){
					((ZPackage)mHiddenRelationship.getSource()).clearRequiredInterfaceSubstitutions(mHiddenRelationship);
					System.out.println(mHiddenRelationship.getSource().getId()+"pois reqinfsubs"+mHiddenRelationship.getDestination().getId());
				    }
				    else{
					((ZVisualLeafModelElement)mHiddenRelationship.getSource()).clearRequiredInterfaceSubstitutions(mHiddenRelationship);
					System.out.println(mHiddenRelationship.getSource().getId()+"pois reqinfsubs"+mHiddenRelationship.getDestination().getId());
				    }
				}
                            }
			}

			mOffInterface.clearHiddenRelationships();
			mOffInterface.removeSubsDeps();
		    }
		    if(mSelection[i] instanceof ZPackage){
			((ZPackage)mSelection[i]).clearHiddenOfferedRealizations();
			((ZPackage)mSelection[i]).clearOfferedRealizationSubstitutions();
		    }
		    else{
			((ZVisualLeafModelElement)mSelection[i]).clearHiddenOfferedRealizations();
                        ((ZVisualLeafModelElement)mSelection[i]).clearOfferedRealizationSubstitutions();
		    }
		}
	}

        /**
	 * Private variable to hold the ModelStorage instance that is needed
	 * in various places in the recursive action of this object. 
	 */
	private ModelStorage  mModel		= null;
	
	/**
	 * Private variable to hold the Visualization instance that is needed
	 * in various places in the recursive action of this object. 
	 */
	private Visualization mVisualization	= null;

}
