/**
 * @(#) VInterface.java
 */

package venice;

import java.util.LinkedList;

/**
 * VInterface represents an interface in the software architecture
 * <p>
 * Interface has operations and a type and it can be visualized
 * in two different modes: either as a circle or like class
 *
 * @author Hannu Laurila
 * @author Antti Pietarinen
 */
public interface VInterface extends VModelElement
{
    /** a constant describing that this interface is an offered interface */
    public static final int TYPE_OFFERED=1;

    /** a constant describing that this interface is a required interface */
    public static final int TYPE_REQUIRED=2;

    /** a constant: mode when interface is visualized as a circle */
    public static final int VIEWMODE_CIRCLE=1;

    /** a constant: mode when interface is visualized like class */
    public static final int VIEWMODE_EXPANDED=2;

    /** Set inElement to hidden relationships list.
     */
    public void setHiddenRelationships(VModelElement inElement);
    /** Get content of hidden relationships.*/
    public LinkedList getHiddenRelationships();
    /** Clear list of hidden relationship.*/
    public void clearHiddenRelationships();
    /** Add inElement to list of dependencies that are greated 
     * to substitute this interface.
     */
    public void addSubsDeps(VModelElement inElement);
    /**
     * Get content of substitute dependency list.
     */
    public LinkedList  getSubsDeps();
    /**
       Clear substitute dependency list.
     */
    public void  removeSubsDeps();

    /**
     * returns the type of interface (offered or required)
     *
     * @return one of TYPE_ constants
     */
    public int getInterfaceType();
    
    /**
     * returns the array which contains the operations in this interface.
     * <p>
     * The array has been given to VInterface using setOperations method.
     * See setOperations method description for information on the array
     * content
     */
    public Object[] getOperations();
    
    /** 
     * returns the current view mode of the interface.
     *
     * @return one of VIEWMODE_ constants
     */
    public int getViewMode();
    
    /**
     * sets the type of the interface (offered or required).
     *
     * The type affects the way how the interface is visualized.
     * An offered interface is visualised as a filled circle and a
     * required interface is visualized as an empty circle
     */
    public void setInterfaceType(int inType);
    
    /**
     * sets the operations of the interface.
     * <p>
     * The operations are shown when the interface is visualized in
     * expanded mode.
     *
     * @param inOps the array of objects representing the operations
     *              of the interface. The objects toString() method is used
     *              to get string representation of the operations
     */
    public void setOperations(Object[] inOps);
    
    /**
     * sets the view mode of the interface.
     *
     * @param inMode one of VIEWMODE_ constants
     */
    public void setViewMode(int inMode);
}
