/**
 * @(#) VPackage.java
 */

package venice;

import java.util.LinkedList;

/**
 * VPackage is a visual component representing a package in the
 * software architecture. A package has childs which are other model
 * elements including other components, interfaces and other
 * packages.
 *
 * @author Hannu Laurila
 */
public interface VPackage extends VModelElement
{
    public static final int CLOSED = 1;
    public static final int OPEN = 2;
    public static final int TRANSPARENT = 3;
    
    
    /**
     * Adds a modelelement to be children of this package.
     * The model element is visualized to be inside the package.
     */
    public void addVChild(VModelElement inElem);

    /**
     * Get current status of package content visibility
     *
     * @return true if contents of the package is shown, false otherwise.
     */
    public boolean getShowContent();
    
    /**
     * returns an array consisting of contents of the package
     */
    public VModelElement[] getVChildren();
    
    /**
     * removes a child from the package
     */
    public void removeVChild(VModelElement inElem);

    /**
     * sets the status of package content visibility
     */
    public void setShowContent(boolean inState);

    /**
     * Sets the state of the package. State can be either CLOSED, OPEN or TRANSPARENT.
     */
    public void setState(int inState);
    
    /**
     * Tells the state of this package.
     */
    public int getState();
    
    //    public void removeSubsRelationship(VRelationship inRea);
    //    public void addSubsRelationship(VRelationship inRea);
}

