/**
 * @(#) VisualizationFactory.java
 */

package venice;

/**
 * VisualizationFactory is a class which returns a concrete
 * implementations of Visualization interface. It is a factory
 * class with only one method for constructing new visualizations.
 * <p>
 * Currently the visualization is based on JAZZ graphics library.
 * If the visualization library is changed, this class can be modified
 * to return instance of the new Visualization system.
 *
 * @author Hannu Laurila
 */

public final class VisualizationFactory      
{
    /**
     * a private constructor which prevents creating instances
     * of this class.
     */
    private VisualizationFactory()
    {
    }

    /**
     * constructs a new concrete instance of Visualization interface.
     */
    public static Visualization newVisualization()
    {
	return new ZVisualization();
    }
}
