/**
 * @(#) ZComponent.java
 */

package venice;

import java.awt.*;
import java.awt.geom.*;
import edu.umd.cs.jazz.*;
import edu.umd.cs.jazz.component.*;
import edu.umd.cs.jazz.util.*;

/**
 * A concrete implementation of VComponent interface using Jazz library.
 *
 * @author Hannu Laurila
 */
public class ZComponent extends ZVisualLeafModelElement implements VComponent
{
	/** Jazz visual component visualizing the component label */
	private ZLabel mComponentLabel;
	
	/** Jazz visual component visualizing the rectangle around the label */
	private ZRectangle mComponentRectangle;
	
	public ZComponent(String inId)
	{
		super(inId);
		this.mComponentLabel = new ZLabel();
		this.mComponentRectangle = new ZRectangle();
		this.mComponentLabel.setTranslation(4,4);
		this.mComponentRectangle.setFillPaint(null);
		this.addVisualComponent(mComponentRectangle);
		this.addVisualComponent(mComponentLabel);
		dataChanged();
	}
    
	protected void dataChanged()
	{
	if (mModelElementVisible == false)
	    this.editor().getInvisibleGroup();
	else
	    this.editor().removeInvisibleGroup();
	
	mComponentLabel.setText(mModelElementName);
	
	ZBounds myLabelBounds = mComponentLabel.getBoundsReference();
	
	double myRectWidth = 4 + myLabelBounds.getWidth() + 4;
	double myRectHeight = 4 + myLabelBounds.getHeight() + 4;
	
	mComponentRectangle.setRect(0, 0, myRectWidth, myRectHeight);	
    }
    
	public Point2D getRelationshipConnectPoint()
	{
		ZBounds myBounds = getGlobalBounds();
		
		return myBounds.getCenter2D();
	}
	
    public Point2D intersectWithLine(Line2D inLine)
    {
		ZBounds myBounds = getGlobalBounds();
		
		return Intersections.lineIntersectsRectangle(inLine, myBounds);
    }
}
